import requests

from . import __version__
from .url import URL, CATALOG, ALL_THREADS
from .thread import Thread

def get_boards(board_name_list, *args, **kwargs):
    if isinstance(board_name_list, basestring):
        board_name_list = board_name_list.split()
    return [Board(name, *args, **kwargs) for name in board_name_list]

class Board(object):
    def __init__(self, board_name, https=False, clean_comments=True,
                 api_url=URL['api'], session=None):
        self._board_name = board_name
        self._protocol = 'https://' if https else 'http://'
        self._clean_comments = clean_comments
        self._base_url = self._protocol + api_url

        self._requests_session = session or requests.session()
        self._requests_session.headers['User-Agent'] = 'py-4chan/%s' % __version__

        self._board_path = '%s/%s' % (self._base_url,
                                      URL['template']['board'].format(name=board_name))
        self._thread_path = '%s/%s' % (self._base_url,
                                       URL['template']['thread'].format(name=board_name))

        self._thread_cache = {}

    def _get_json(self, path):
        res = self._requests_session.get(self._board_path % path)
        res.raise_for_status()
        return res.json()

    def get_thread(self, id, update_if_cached=True):
        """
            Get a thread from 4chan via 4chan API.

            :param id: Thread ID
            :param update_if_cached: Should the thread be updated if it's found in the cache?
            :return: Thread
        """
        if id in self._thread_cache:
            thread = self._thread_cache[id]
            if update_if_cached:
                thread.update()

            return thread

        res = self._requests_session.get(self._thread_path % id)

        thread = Thread._from_request(self, res, id)
        self._thread_cache[id] = thread

        return thread

    def thread_exists(self, id):
        """
            Check if a thread exists, or is 404.
            :param id: Thread ID
            :return: bool
        """
        return self._requests_session.head(self._thread_path % id).ok

    def _catalog_to_threads(self, json):
        threads_json = [thread for page in json for thread in page['threads']]
        thread_list = [{'posts': [thread] + thread.get('last_replies', [])}
                       for thread in threads_json]

        for thread in thread_list:
            thread['posts'][0].pop('last_replies', None)

        return thread_list

    def _request_threads(self, page):
        json = self._get_json(page)

        if page == CATALOG:
            thread_list = self._catalog_to_threads(json)
        else:
            thread_list = json['threads']

        threads = []
        for thread_json in thread_list:
            id = thread_json['posts'][0]['no']
            if id in self._thread_cache:
                thread = self._thread_cache[id]
                thread.want_update = True
            else:
                thread = Thread._from_json(thread_json, self)
                self._thread_cache[thread.id] = thread

            threads.append(thread)

        return threads

    def get_threads(self, page=1):
        """
            Get all threads on a certain page. Pages are now indexed at 1, instead of 0.
            If the thread is already in cache, return cached thread entry.

            Sets thread.want_update to True if the thread is being returned from the cache.
            :param: page: Page to fetch thread from
            :return: list[Thread]
        """
        return self._request_threads(page)

    def get_all_thread_ids(self):
        """
            Return a list of all thread IDs.

            :return: list[int]
        """
        json = self._get_json(ALL_THREADS)
        return [thread['no'] for page in json for thread in page['threads']]

    def get_all_threads(self, expand=False):
        """
            Return all threads on all pages.

            If not expanded, result is same as get_threads run across all board pages,
            with last 3-5 replies included.

            Uses the catalog when not expanding, and uses the flat thread ID listing
            at /{board}/threads.json when expanding for more efficient resource usage.

            If expanded, all data of all threads is returned with no omitted posts.
            :param: expand: Whether or not to expand threads
            :return: list[Thread]
        """
        if not expand:
            return self._request_threads(CATALOG)

        thread_ids = self.get_all_thread_ids()
        threads = [self.get_thread(id) for id in thread_ids]

        return filter(None, threads)

    def refresh_cache(self, if_want_update=False):
        """
            Update all threads currently stored in the cache.
        """
        for thread in self._thread_cache.values():
            if if_want_update:
                if not thread.want_update:
                    continue
            thread.update()

    def clear_cache(self):
        """
            Remove everything currently stored in the cache.
        """
        self._thread_cache.clear()

    @property
    def name(self):
        """
            Board name that this board represents.
            :return: string
        """
        return self._board_name

    def __repr__(self):
        return '<Board /%s/>' % self.name

board = Board



URL = {
    'api': 'a.4cdn.org',
    'boards': 'boards.4chan.org',
    'images': 'i.4cdn.org',
    'thumbs': 't.4cdn.org',
    'template': {
        'board': '{name}/%s.json',
        'thread': '{name}/thread/%s.json'
    }
}

CATALOG = 'catalog'
ALL_THREADS = 'threads'



"""
    py-4chan
    ~~~~~~~~

    Python Wrappers to access 4chan's API.

    This program is free software. It comes without any warranty, to
    the extent permitted by applicable law. You can redistribute it
    and/or modify it under the terms of the Do What The Fuck You Want
    To Public License, Version 2, as published by Sam Hocevar. See
    http://sam.zoy.org/wtfpl/COPYING for more details.

"""

__version__ = '0.4.0'

from .board import Board, board, get_boards
from .thread import Thread
from .post import Post



#!/usr/bin/env python

"""
    py-4chan-setup
    ~~~~~~~~~~~~~~

    Python Wrappers to access 4chan's API.

    This program is free software. It comes without any warranty, to
    the extent permitted by applicable law. You can redistribute it
    and/or modify it under the terms of the Do What The Fuck You Want
    To Public License, Version 2, as published by Sam Hocevar. See
    http://sam.zoy.org/wtfpl/COPYING for more details.

"""

from setuptools import setup
from py4chan import __version__

setup(name='BASC-py4chan',
    version=__version__,
    description='Python 4chan API Wrapper. Improved version of Edgeworth\'s original py-4chan wrapper.',
    long_description = open('README.rst').read(), 
    license=open('LICENSE').read(),
    author='Antonizoon (Lawrence Wu)',
    author_email='sagnessagiel@gmail.com',
    url='http://github.com/bibanon/BASC-py4chan',
    packages=['py4chan'],
    install_requires = ['requests >= 1.0.0']
)


from datetime import datetime

from .url import URL
from .util import clean_comment_body

class Post(object):
    def __init__(self, thread, data):
        self._thread = thread
        self._data = data
        self._clean_comments = thread._board._clean_comments

    @property
    def post_number(self):
        """
            :return: int
        """
        return self._data['no']
    number = num = no = post_number

    @property
    def id(self):
        """
            :return: int
        """
        return self._data.get('id')

    @property
    def name(self):
        return self._data.get('name')

    @property
    def email(self):
        return self._data.get('email')

    @property
    def tripcode(self):
        return self._data.get('trip')

    @property
    def subject(self):
        return self._data.get('sub')

    @property
    def orig_comment(self):
        return self._data.get('com', '')
    html_comment = orig_comment

    @property
    def comment(self):
        if not self._clean_comments or not self.orig_comment:
            return self.orig_comment

        if 'cleaned_comment' not in self._data:
            self._data['cleaned_comment'] = clean_comment_body(self.orig_comment)

        return self._data['cleaned_comment']
    body = text = comment

    @property
    def timestamp(self):
        return self._data['time']

    @property
    def datetime(self):
        return datetime.fromtimestamp(self._data['time'])

    @property
    def file_md5(self):
        if not self.has_file:
            return None

        return self._data['md5'].decode('base64')

    @property
    def file_md5_hex(self):
        if not self.has_file:
            return None

        return self.file_md5.encode('hex')

    @property
    def filename(self):
        if not self.has_file:
            return None

        board = self._thread._board
        
        return '%i%s' % (
            self._data['tim'],
            self._data['ext']
        )

    @property
    def file_url(self):
        if not self.has_file:
            return None

        board = self._thread._board
        return '%s%s/%s/%i%s' % (
            board._protocol,
            URL['images'],
            board.name,
            self._data['tim'],
            self._data['ext']
        )

    @property
    def file_extension(self):
        return self._data.get('ext')

    @property
    def file_size(self):
        return self._data.get('fsize')

    @property
    def file_width(self):
        return self._data.get('w')

    @property
    def file_height(self):
        return self._data.get('h')

    @property
    def file_deleted(self):
        return self._data.get('filedeleted') == 1

    @property
    def thumbnail_width(self):
        return self._data.get('tn_w')

    @property
    def thumbnail_height(self):
        return self._data.get('tn_h')

    @property
    def thumbnail_fname(self):
        if not self.has_file:
            return None

        board = self._thread._board

        return '%is.jpg' % (
            self._data['tim']
        )

    @property
    def thumbnail_url(self):
        if not self.has_file:
            return None

        board = self._thread._board
        return '%s%s/%s/%is.jpg' % (
            board._protocol,
            URL['thumbs'],
            board.name,
            self._data['tim']
        )

    @property
    def has_file(self):
        return 'filename' in self._data

    def file_request(self):
        return self._thread._board._requests_session.get(self.file_url)

    def thumbnail_request(self):
        return self._thread._board._requests_session.get(self.thumbnail_url)

    @property
    def _semantic_url(self):
        return self._data.get('semantic_url')

    @property
    def post_url(self):
        return '%s#p%i' % (self._thread.thread_url, self.post_number)
    url = post_url

    def __repr__(self):
        return '<Post /%s/%i#%i, has_file: %r>' % (
            self._thread._board.name,
            self._thread.id,
            self.post_number,
            self.has_file
        )
